@@ -47,7 +47,8 @@ typedef enum
     POPPLER_ERROR_ENCRYPTED,
     POPPLER_ERROR_OPEN_FILE,
     POPPLER_ERROR_BAD_CATALOG,
-    POPPLER_ERROR_DAMAGED
+    POPPLER_ERROR_DAMAGED,
+    POPPLER_ERROR_SIGNING
 } PopplerError;
 
 /**
@@ -185,6 +186,7 @@ typedef struct _PopplerPoint PopplerPoint;
 typedef struct _PopplerRectangle PopplerRectangle;
 typedef struct _PopplerTextAttributes PopplerTextAttributes;
 typedef struct _PopplerColor PopplerColor;
+typedef struct _PopplerFontDescription PopplerFontDescription;
 typedef struct _PopplerLinkMapping PopplerLinkMapping;
 typedef struct _PopplerPageTransition PopplerPageTransition;
 typedef struct _PopplerImageMapping PopplerImageMapping;
@@ -219,6 +221,9 @@ typedef struct _PopplerStructureElementIter PopplerStructureElementIter;
 typedef struct _PopplerTextSpan PopplerTextSpan;
 typedef struct _PopplerPageRange PopplerPageRange;
 typedef struct _PopplerSignatureInfo PopplerSignatureInfo;
+typedef struct _PopplerAnnotStamp PopplerAnnotStamp;
+typedef struct _PopplerCertificateInfo PopplerCertificateInfo;
+typedef struct _PopplerSigningData PopplerSigningData;
 
 /**
  * PopplerBackend:
