@@ -40,6 +40,7 @@
 // Copyright (C) 2024 Nelson Benítez León <nbenitezl@gmail.com>
 // Copyright (C) 2024 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 // Copyright (C) 2024 Vincent Lefevre <vincent@vinc17.net>
+// Copyright (C) 2024 G B <glen.browman@veeva.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -186,17 +187,6 @@ GfxFontLoc::GfxFontLoc(GfxFontLoc &&other) noexcept = default;
 
 GfxFontLoc &GfxFontLoc::operator=(GfxFontLoc &&other) noexcept = default;
 
-void GfxFontLoc::setPath(GooString *pathA)
-{
-    path = pathA->toStr();
-    delete pathA;
-}
-
-const GooString *GfxFontLoc::pathAsGooString() const
-{
-    return (const GooString *)(&path);
-}
-
 //------------------------------------------------------------------------
 // GfxFont
 //------------------------------------------------------------------------
@@ -236,14 +226,11 @@ std::unique_ptr<GfxFont> GfxFont::makeFont(XRef *xref, const char *tagA, Ref idA
     typeA = getFontType(xref, fontDict, &embFontIDA);
 
     // create the font object
-    GfxFont *font;
     if (typeA < fontCIDType0) {
-        font = new Gfx8BitFont(xref, tagA, idA, std::move(name), typeA, embFontIDA, fontDict);
+        return std::make_unique<Gfx8BitFont>(xref, tagA, idA, std::move(name), typeA, embFontIDA, fontDict);
     } else {
-        font = new GfxCIDFont(xref, tagA, idA, std::move(name), typeA, embFontIDA, fontDict);
+        return std::make_unique<GfxCIDFont>(xref, tagA, idA, std::move(name), typeA, embFontIDA, fontDict);
     }
-
-    return std::unique_ptr<GfxFont>(font);
 }
 
 GfxFont::GfxFont(const char *tagA, Ref idA, std::optional<std::string> &&nameA, GfxFontType typeA, Ref embFontIDA) : tag(tagA), id(idA), name(std::move(nameA)), type(typeA)
@@ -257,13 +244,7 @@ GfxFont::GfxFont(const char *tagA, Ref idA, std::optional<std::string> &&nameA,
     hasToUnicode = false;
 }
 
-GfxFont::~GfxFont()
-{
-    delete family;
-    if (embFontName) {
-        delete embFontName;
-    }
-}
+GfxFont::~GfxFont() = default;
 
 bool GfxFont::isSubset() const
 {
@@ -495,13 +476,13 @@ void GfxFont::readFontDescriptor(XRef *xref, Dict *fontDict)
         // get name
         obj2 = obj1.dictLookup("FontName");
         if (obj2.isName()) {
-            embFontName = new GooString(obj2.getName());
+            embFontName = std::make_unique<GooString>(obj2.getName());
         }
         if (embFontName == nullptr) {
             // get name with typo
             obj2 = obj1.dictLookup("Fontname");
             if (obj2.isName()) {
-                embFontName = new GooString(obj2.getName());
+                embFontName = std::make_unique<GooString>(obj2.getName());
                 error(errSyntaxWarning, -1, "The file uses Fontname instead of FontName please notify the creator that the file is broken");
             }
         }
@@ -509,7 +490,7 @@ void GfxFont::readFontDescriptor(XRef *xref, Dict *fontDict)
         // get family
         obj2 = obj1.dictLookup("FontFamily");
         if (obj2.isString()) {
-            family = new GooString(obj2.getString());
+            family = std::make_unique<GooString>(obj2.getString());
         }
 
         // get stretch
@@ -610,31 +591,30 @@ void GfxFont::readFontDescriptor(XRef *xref, Dict *fontDict)
     }
 }
 
-CharCodeToUnicode *GfxFont::readToUnicodeCMap(Dict *fontDict, int nBits, CharCodeToUnicode *ctu)
+std::unique_ptr<CharCodeToUnicode> GfxFont::readToUnicodeCMap(Dict *fontDict, int nBits, std::unique_ptr<CharCodeToUnicode> ctu)
 {
-    GooString *buf;
 
     Object obj1 = fontDict->lookup("ToUnicode");
     if (!obj1.isStream()) {
-        return nullptr;
+        return ctu;
     }
-    buf = new GooString();
-    obj1.getStream()->fillGooString(buf);
+    auto buf = std::make_unique<GooString>();
+    obj1.getStream()->fillGooString(buf.get());
     obj1.streamClose();
     if (ctu) {
-        ctu->mergeCMap(buf, nBits);
+        ctu->mergeCMap(buf.get(), nBits);
     } else {
-        ctu = CharCodeToUnicode::parseCMap(buf, nBits);
+        ctu = CharCodeToUnicode::parseCMap(buf.get(), nBits);
     }
     hasToUnicode = true;
-    delete buf;
     return ctu;
 }
 
 std::optional<GfxFontLoc> GfxFont::locateFont(XRef *xref, PSOutputDev *ps, GooString *substituteFontName)
 {
     SysFontType sysFontType;
-    GooString *path, *base14Name;
+    std::optional<std::string> path;
+    GooString *base14Name;
     int substIdx, fontNum;
     bool embed;
 
@@ -705,7 +685,7 @@ std::optional<GfxFontLoc> GfxFont::locateFont(XRef *xref, PSOutputDev *ps, GooSt
 
     //----- external font file (fontFile, fontDir)
     if (name && (path = globalParams->findFontFile(*name))) {
-        if (std::optional<GfxFontLoc> fontLoc = getExternalFont(path, isCIDFont())) {
+        if (std::optional<GfxFontLoc> fontLoc = getExternalFont(*path, isCIDFont())) {
             return fontLoc;
         }
     }
@@ -714,7 +694,7 @@ std::optional<GfxFontLoc> GfxFont::locateFont(XRef *xref, PSOutputDev *ps, GooSt
     if (!ps && !isCIDFont() && ((Gfx8BitFont *)this)->base14) {
         base14Name = new GooString(((Gfx8BitFont *)this)->base14->base14Name);
         if ((path = globalParams->findBase14FontFile(base14Name, this, substituteFontName))) {
-            if (std::optional<GfxFontLoc> fontLoc = getExternalFont(path, false)) {
+            if (std::optional<GfxFontLoc> fontLoc = getExternalFont(*path, false)) {
                 delete base14Name;
                 return fontLoc;
             }
@@ -729,13 +709,13 @@ std::optional<GfxFontLoc> GfxFont::locateFont(XRef *xref, PSOutputDev *ps, GooSt
                 GfxFontLoc fontLoc;
                 fontLoc.locType = gfxFontLocExternal;
                 fontLoc.fontType = fontCIDType2;
-                fontLoc.setPath(path);
+                fontLoc.path = *path;
                 fontLoc.fontNum = fontNum;
                 return fontLoc;
             }
         } else {
             GfxFontLoc fontLoc;
-            fontLoc.setPath(path);
+            fontLoc.path = *path;
             fontLoc.locType = gfxFontLocExternal;
             if (sysFontType == sysFontTTF || sysFontType == sysFontTTC) {
                 fontLoc.fontType = fontTrueType;
@@ -745,7 +725,6 @@ std::optional<GfxFontLoc> GfxFont::locateFont(XRef *xref, PSOutputDev *ps, GooSt
             }
             return fontLoc;
         }
-        delete path;
     }
 
     if (!isCIDFont()) {
@@ -776,7 +755,7 @@ std::optional<GfxFontLoc> GfxFont::locateFont(XRef *xref, PSOutputDev *ps, GooSt
         } else {
             path = globalParams->findFontFile(substName);
             if (path) {
-                if (std::optional<GfxFontLoc> fontLoc = getExternalFont(path, false)) {
+                if (std::optional<GfxFontLoc> fontLoc = getExternalFont(*path, false)) {
                     error(errSyntaxWarning, -1, "Substituting font '{0:s}' for '{1:s}'", base14SubstFonts[substIdx], name ? name->c_str() : "");
                     name = base14SubstFonts[substIdx];
                     fontLoc->substIdx = substIdx;
@@ -793,12 +772,12 @@ std::optional<GfxFontLoc> GfxFont::locateFont(XRef *xref, PSOutputDev *ps, GooSt
     return std::nullopt;
 }
 
-std::optional<GfxFontLoc> GfxFont::getExternalFont(GooString *path, bool cid)
+std::optional<GfxFontLoc> GfxFont::getExternalFont(const std::string &path, bool cid)
 {
     FoFiIdentifierType fft;
     GfxFontType fontType;
 
-    fft = FoFiIdentifier::identifyFile(path->c_str());
+    fft = FoFiIdentifier::identifyFile(path.c_str());
     switch (fft) {
     case fofiIdType1PFA:
     case fofiIdType1PFB:
@@ -827,13 +806,12 @@ std::optional<GfxFontLoc> GfxFont::getExternalFont(GooString *path, bool cid)
         break;
     }
     if (fontType == fontUnknownType || (cid ? (fontType < fontCIDType0) : (fontType >= fontCIDType0))) {
-        delete path;
         return std::nullopt;
     }
     GfxFontLoc fontLoc;
     fontLoc.locType = gfxFontLocExternal;
     fontLoc.fontType = fontType;
-    fontLoc.setPath(path);
+    fontLoc.path = path;
     return fontLoc;
 }
 
@@ -984,8 +962,6 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, std::optional<st
     Object obj1;
     int n, a, b, m;
 
-    ctu = nullptr;
-
     // do font name substitution for various aliases of the Base 14 font
     // names
     base14 = nullptr;
@@ -1109,7 +1085,11 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, std::optional<st
     baseEnc = nullptr;
     baseEncFromFontFile = false;
     obj1 = fontDict->lookup("Encoding");
-    if (obj1.isDict()) {
+    bool isZapfDingbats = name && name->ends_with("ZapfDingbats");
+    if (isZapfDingbats) {
+        baseEnc = zapfDingbatsEncoding;
+        hasEncoding = true;
+    } else if (obj1.isDict()) {
         Object obj2 = obj1.dictLookup("BaseEncoding");
         if (obj2.isName("MacRomanEncoding")) {
             hasEncoding = true;
@@ -1145,8 +1125,7 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, std::optional<st
             if ((ffT1 = FoFiType1::make(buf->data(), buf->size()))) {
                 const std::string fontName = ffT1->getName();
                 if (!fontName.empty()) {
-                    delete embFontName;
-                    embFontName = new GooString(fontName);
+                    embFontName = std::make_unique<GooString>(fontName);
                 }
                 if (!baseEnc) {
                     baseEnc = (const char **)ffT1->getEncoding();
@@ -1159,10 +1138,7 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, std::optional<st
         if (buf) {
             if ((ffT1C = FoFiType1C::make(buf->data(), buf->size()))) {
                 if (ffT1C->getName()) {
-                    if (embFontName) {
-                        delete embFontName;
-                    }
-                    embFontName = new GooString(ffT1C->getName());
+                    embFontName = std::make_unique<GooString>(ffT1C->getName());
                 }
                 if (!baseEnc) {
                     baseEnc = (const char **)ffT1C->getEncoding();
@@ -1254,7 +1230,6 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, std::optional<st
 
     // pass 1: use the name-to-Unicode mapping table
     missing = hex = false;
-    bool isZapfDingbats = name && name->ends_with("ZapfDingbats");
     for (int code = 0; code < 256; ++code) {
         if ((charName = enc[code])) {
             if (isZapfDingbats) {
@@ -1326,7 +1301,7 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, std::optional<st
     // existing entries in ctu, i.e., the ToUnicode CMap takes
     // precedence, but the other encoding info is allowed to fill in any
     // holes
-    readToUnicodeCMap(fontDict, 16, ctu);
+    ctu = readToUnicodeCMap(fontDict, 16, std::move(ctu));
 
     //----- get the character widths -----
 
@@ -1426,7 +1401,6 @@ Gfx8BitFont::~Gfx8BitFont()
             gfree(enc[i]);
         }
     }
-    ctu->decRefCnt();
 }
 
 // This function is in part a derived work of the Adobe Glyph Mapping
@@ -1557,7 +1531,7 @@ int Gfx8BitFont::getNextChar(const char *s, int len, CharCode *code, Unicode con
 
 const CharCodeToUnicode *Gfx8BitFont::getToUnicode() const
 {
-    return ctu;
+    return ctu.get();
 }
 
 int *Gfx8BitFont::getCodeToGIDMap(FoFiTrueType *ff)
@@ -1740,7 +1714,6 @@ GfxCIDFont::GfxCIDFont(XRef *xref, const char *tagA, Ref idA, std::optional<std:
     descent = -0.35;
     fontBBox[0] = fontBBox[1] = fontBBox[2] = fontBBox[3] = 0;
     collection = nullptr;
-    ctu = nullptr;
     ctuUsesCharCode = true;
     widths.defWidth = 1.0;
     widths.defHeight = -1.0;
@@ -1777,11 +1750,11 @@ GfxCIDFont::GfxCIDFont(XRef *xref, const char *tagA, Ref idA, std::optional<std:
             obj2 = Object(new GooString("Adobe"));
             obj3 = Object(new GooString("Identity"));
         }
-        collection = obj2.getString()->copy()->append('-')->append(obj3.getString());
+        collection.reset(obj2.getString()->copy()->append('-')->append(obj3.getString()));
     } else {
         error(errSyntaxError, -1, "Missing CIDSystemInfo dictionary in Type 0 descendant font");
         error(errSyntaxError, -1, "Assuming Adobe-Identity for character collection");
-        collection = new GooString("Adobe-Identity");
+        collection = std::make_unique<GooString>("Adobe-Identity");
     }
 
     // look for a ToUnicode CMap
@@ -1794,7 +1767,7 @@ GfxCIDFont::GfxCIDFont(XRef *xref, const char *tagA, Ref idA, std::optional<std:
             ctu = CharCodeToUnicode::makeIdentityMapping();
         } else {
             // look for a user-supplied .cidToUnicode file
-            if (!(ctu = globalParams->getCIDToUnicode(collection))) {
+            if (!(ctu = globalParams->getCIDToUnicode(collection.get()))) {
                 // I'm not completely sure that this is the best thing to do
                 // but it seems to produce better results when the .cidToUnicode
                 // files from the poppler-data package are missing. At least
@@ -1805,11 +1778,11 @@ GfxCIDFont::GfxCIDFont(XRef *xref, const char *tagA, Ref idA, std::optional<std:
                 };
                 for (const char *knownCollection : knownCollections) {
                     if (collection->cmp(knownCollection) == 0) {
-                        error(errSyntaxError, -1, "Missing language pack for '{0:t}' mapping", collection);
+                        error(errSyntaxError, -1, "Missing language pack for '{0:t}' mapping", collection.get());
                         return;
                     }
                 }
-                error(errSyntaxError, -1, "Unknown character collection '{0:t}'", collection);
+                error(errSyntaxError, -1, "Unknown character collection '{0:t}'", collection.get());
                 // fall-through, assuming the Identity mapping -- this appears
                 // to match Adobe's behavior
             }
@@ -1822,7 +1795,7 @@ GfxCIDFont::GfxCIDFont(XRef *xref, const char *tagA, Ref idA, std::optional<std:
         error(errSyntaxError, -1, "Missing Encoding entry in Type 0 font");
         return;
     }
-    if (!(cMap = CMap::parse(nullptr, collection, &obj1))) {
+    if (!(cMap = CMap::parse(nullptr, collection.get(), &obj1))) {
         return;
     }
     if (cMap->getCMapName()) {
@@ -1949,12 +1922,6 @@ GfxCIDFont::GfxCIDFont(XRef *xref, const char *tagA, Ref idA, std::optional<std:
 
 GfxCIDFont::~GfxCIDFont()
 {
-    if (collection) {
-        delete collection;
-    }
-    if (ctu) {
-        ctu->decRefCnt();
-    }
     if (cidToGID) {
         gfree(cidToGID);
     }
@@ -2036,7 +2003,7 @@ int GfxCIDFont::getWMode() const
 
 const CharCodeToUnicode *GfxCIDFont::getToUnicode() const
 {
-    return ctu;
+    return ctu.get();
 }
 
 const GooString *GfxCIDFont::getCollection() const
@@ -2171,10 +2138,9 @@ int *GfxCIDFont::getCodeToGIDMap(FoFiTrueType *ff, int *codeToGIDLen)
     humap = new Unicode[n * N_UCS_CANDIDATES];
     memset(humap, 0, sizeof(Unicode) * n * N_UCS_CANDIDATES);
     if (lp->collection != nullptr) {
-        CharCodeToUnicode *tctu;
         GooString tname(lp->toUnicodeMap);
 
-        if ((tctu = CharCodeToUnicode::parseCMapFromFile(&tname, 16)) != nullptr) {
+        if (std::unique_ptr<CharCodeToUnicode> tctu = CharCodeToUnicode::parseCMapFromFile(&tname, 16)) {
             tumap = new Unicode[n];
             CharCode cid;
             for (cid = 0; cid < n; cid++) {
@@ -2189,7 +2155,6 @@ int *GfxCIDFont::getCodeToGIDMap(FoFiTrueType *ff, int *codeToGIDLen)
                     tumap[cid] = 0;
                 }
             }
-            delete tctu;
         }
         vumap = new Unicode[n];
         memset(vumap, 0, sizeof(Unicode) * n);
@@ -2328,7 +2293,7 @@ double GfxCIDFont::getWidth(char *s, int len) const
 // GfxFontDict
 //------------------------------------------------------------------------
 
-GfxFontDict::GfxFontDict(XRef *xref, Ref *fontDictRef, Dict *fontDict)
+GfxFontDict::GfxFontDict(XRef *xref, const Ref fontDictRef, Dict *fontDict)
 {
     Ref r;
 
@@ -2339,10 +2304,10 @@ GfxFontDict::GfxFontDict(XRef *xref, Ref *fontDictRef, Dict *fontDict)
         if (obj2.isDict()) {
             if (obj1.isRef()) {
                 r = obj1.getRef();
-            } else if (fontDictRef) {
+            } else if (fontDictRef != Ref::INVALID()) {
                 // legal generation numbers are five digits, so we use a
                 // 6-digit number here
-                r.gen = 100000 + fontDictRef->num;
+                r.gen = 100000 + fontDictRef.num;
                 r.num = i;
             } else {
                 // no indirect reference for this font, or for the containing
